/*global AppMobi, XMLHttpRequest, ActiveXObject */

var AjaxBridge = {};

/**
 * Send XML HTTP Request
 * @param {string} method
 * @param {string} url
 * @param {boolean} async
 * @param {Number} timeout
 * @param {string} params - already serialized params
 * @param {string} callback - callback function name, the function should accepts the following params: url, status, response
 * @param {string} responseType
 */
AjaxBridge.send = function (method, url, async, timeout, params, callback, responseType) { "use strict";
    var xhr = AjaxBridge.createRequest();
    if (!xhr) {
        return;
    }
    xhr.timeout = timeout || 0;
    xhr.open(method, url, async);
    if (method === AjaxBridge.METHOD.POST) {
        // send the proper header information along with the request
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    }
    if (async) {
        xhr.onreadystatechange = function() {
            if (xhr.readyState === AjaxBridge.STATE.DONE) {
                AppMobi.canvas.execute(callback + "('" + url + "', "  + xhr.status + ", '" + AjaxBridge.getResponse(xhr, responseType) + "');");
            }
        };
    }
    xhr.send(params);
    if (!async) {
        AppMobi.canvas.execute(callback + "('" + url + "', "  + xhr.status + ", '" + AjaxBridge.getResponse(xhr, responseType) + "');");
    }
};

AjaxBridge.createRequest = function() { "use strict";
    if (window.XMLHttpRequest) {
        return new XMLHttpRequest();
    } else if (window.ActiveXObject) {
        try {
            return new ActiveXObject("Msxml2.XMLHTTP");
        } catch (e1) {
            try {
                return new ActiveXObject("Microsoft.XMLHTTP");
            } catch (e2) {
                return null;
            }
        }
    }
    return null;
};

AjaxBridge.getResponse = function(xhr, responseType) { "use strict";
    switch (responseType) {
        case AjaxBridge.RESPONSE_TYPE.TEXT:
            return xhr.responseText;
        case AjaxBridge.RESPONSE_TYPE.JSON:
            return xhr.responseText;
        case AjaxBridge.RESPONSE_TYPE.XML:
            return JSON.stringify(AjaxBridge.xml2json(AjaxBridge.parseXml(xhr.responseText)));
        default:
            return xhr.response;
    }
};

AjaxBridge.parseXml = function(xml) { "use strict";
    var dom = null;
    if (window.DOMParser) {
        try {
            dom = (new DOMParser()).parseFromString(xml, "text/xml");
        } catch (e) {
            // parse error
            dom = null;
        }
    } else if (window.ActiveXObject) {
        try {
            dom = new ActiveXObject('Microsoft.XMLDOM');
            dom.async = false;
            if (!dom.loadXML(xml)){
                // parse error
                dom = null;
            }
        } catch (e) {
            // parse error
            dom = null;
        }
    } else {
        // no parser
        dom = null;
    }
    return dom;
};

AjaxBridge.xml2json = function(xml) { "use strict";
    var obj = {};
    if (xml.nodeType === 1) {
        // element, do attributes
        if (xml.attributes.length > 0) {
            obj["@attributes"] = {};
            for (var j = 0; j < xml.attributes.length; j++) {
                var attribute = xml.attributes.item(j);
                obj["@attributes"][attribute.nodeName] = attribute.nodeValue;
            }
        }
    } else if (xml.nodeType === 3) {
        // text, replace line breaks with spaces
        obj = xml.nodeValue.replace(/(\r\n|\n|\r)/gm," ");
    }
    if (xml.hasChildNodes()) {
        for (var i = 0; i < xml.childNodes.length; i++) {
            var item = xml.childNodes.item(i);
            var nodeName = item.nodeName;
            if (obj[nodeName] === undefined) {
                obj[nodeName] = AjaxBridge.xml2json(item);
            } else {
                if (obj[nodeName].push === undefined) {
                    var old = obj[nodeName];
                    obj[nodeName] = [];
                    obj[nodeName].push(old);
                }
                obj[nodeName].push(AjaxBridge.xml2json(item));
            }
        }
    }
    return obj;
};

AjaxBridge.METHOD = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    DELETE: 'DELETE'
};

AjaxBridge.STATE = {
    UNSENT: 0, // open() has not been called yet
    OPENED: 1, // send() has not been called yet
    HEADERS_RECEIVED: 2, // send() has been called, headers and status are available
    LOADING: 3, // downloading, responseText holds partial data
    DONE: 4 // the operation has completed
};

AjaxBridge.RESPONSE_TYPE = {
    TEXT: 'text',
    JSON: 'json',
    XML: 'document'
};
